import { useState, useMemo } from 'react'
import { toast } from 'react-toastify'
import { REACTIONS } from 'constants.ts'
import { RelayController, UserRelaysType } from 'controllers'
import { useAppSelector, useDidMount } from 'hooks'
import { Event, Filter, UnsignedEvent, kinds } from 'nostr-tools'
import { abbreviateNumber, log, LogType, now } from 'utils'

type UseReactionsParams = {
  pubkey: string
  eTag: string
  aTag?: string
}

export const useReactions = (params: UseReactionsParams) => {
  const [isReactionInProgress, setIsReactionInProgress] = useState(false)
  const [isDataLoaded, setIsDataLoaded] = useState(false)
  const [reactionEvents, setReactionEvents] = useState<Event[]>([])

  const userState = useAppSelector((state) => state.user)

  useDidMount(() => {
    const filter: Filter = {
      kinds: [kinds.Reaction]
    }

    if (params.aTag) {
      filter['#a'] = [params.aTag]
    } else {
      filter['#e'] = [params.eTag]
    }

    RelayController.getInstance()
      .fetchEventsFromUserRelays(filter, params.pubkey, UserRelaysType.Read)
      .then((events) => {
        setReactionEvents(events)
      })
      .finally(() => {
        setIsDataLoaded(true)
      })
  })

  const hasReactedPositively = useMemo(() => {
    return (
      !!userState.auth &&
      reactionEvents.some(
        (event) =>
          event.pubkey === userState.user?.pubkey &&
          (REACTIONS.positive.emojis.includes(event.content) ||
            REACTIONS.positive.shortCodes.includes(event.content))
      )
    )
  }, [reactionEvents, userState])

  const hasReactedNegatively = useMemo(() => {
    return (
      !!userState.auth &&
      reactionEvents.some(
        (event) =>
          event.pubkey === userState.user?.pubkey &&
          (REACTIONS.negative.emojis.includes(event.content) ||
            REACTIONS.negative.shortCodes.includes(event.content))
      )
    )
  }, [reactionEvents, userState])

  const getPubkey = async () => {
    let hexPubkey: string

    if (userState.auth && userState.user?.pubkey) {
      hexPubkey = userState.user.pubkey as string
    } else {
      hexPubkey = (await window.nostr?.getPublicKey()) as string
    }

    if (!hexPubkey) {
      toast.error('Could not get pubkey')
      return null
    }

    return hexPubkey
  }

  const handleReaction = async (isPositive?: boolean) => {
    if (!isDataLoaded || hasReactedPositively || hasReactedNegatively) return

    if (isReactionInProgress) return

    setIsReactionInProgress(true)

    try {
      const pubkey = await getPubkey()
      if (!pubkey) return

      const unsignedEvent: UnsignedEvent = {
        kind: kinds.Reaction,
        created_at: now(),
        content: isPositive ? '+' : '-',
        pubkey,
        tags: [
          ['e', params.eTag],
          ['p', params.pubkey]
        ]
      }

      if (params.aTag) {
        unsignedEvent.tags.push(['a', params.aTag])
      }

      const signedEvent = await window.nostr
        ?.signEvent(unsignedEvent)
        .then((event) => event as Event)
        .catch((err) => {
          toast.error('Failed to sign the reaction event!')
          log(true, LogType.Error, 'Failed to sign the event!', err)
          return null
        })

      if (!signedEvent) return

      setReactionEvents((prev) => [...prev, signedEvent])

      const publishedOnRelays = await RelayController.getInstance().publish(
        signedEvent as Event,
        params.pubkey,
        UserRelaysType.Read
      )

      if (publishedOnRelays.length === 0) {
        log(
          true,
          LogType.Error,
          'Failed to publish reaction event on any relay'
        )
        return
      }
    } finally {
      setIsReactionInProgress(false)
    }
  }

  const { likesCount, disLikesCount } = useMemo(() => {
    let positiveCount = 0
    let negativeCount = 0

    reactionEvents.forEach((event) => {
      if (
        REACTIONS.positive.emojis.includes(event.content) ||
        REACTIONS.positive.shortCodes.includes(event.content)
      ) {
        positiveCount++
      } else if (
        REACTIONS.negative.emojis.includes(event.content) ||
        REACTIONS.negative.shortCodes.includes(event.content)
      ) {
        negativeCount++
      }
    })

    return {
      likesCount: abbreviateNumber(positiveCount),
      disLikesCount: abbreviateNumber(negativeCount)
    }
  }, [reactionEvents])

  return {
    isDataLoaded,
    likesCount,
    disLikesCount,
    hasReactedPositively,
    hasReactedNegatively,
    handleReaction
  }
}
